//********************************************************************
UNIT GraphicSupport;
//********************************************************************
INTERFACE

USES
   SysUtils, Classes, Math, dglOpenGL,
   StdBasicDefs, OSSupport, Math3D;

TYPE
   TVertexBuffer = Array Of TVector3D;
   TIndexBuffer = Array Of TuInt16;


// This is set by the main program
Procedure SetGraphicWindow(AWindow: TuInt32);
Procedure ResizeGraphicWindow(AWidth,AHeight: TuInt32);

// This is used by shapeman
Procedure InitGraphicParams(AZNear,AZFar: TInt32);
Procedure CreateGraphicTools;
Procedure FreeGraphicTools;
Procedure SetViewMatrix(AViewMatrix: TMatrix4D);

Procedure BeginRender;
Procedure EndRender;
Procedure Render(AMatrix: TMatrix4D; AColor: T32bColor;
                 AVertexBuffer,ANormalBuffer: TVertexBuffer;
                 AIndexBuffer: TIndexBuffer);
//********************************************************************
IMPLEMENTATION
//********************************************************************
CONST
   Mat_Specular   : Array[0..3] Of TFloat32 = (0.8, 0.8, 0.8, 1.0);
   Mat_Shininess  : Array[0..0] Of TFloat32 = (50.0);
   Mat_Ambient    : Array[0..3] Of TFloat32 = (0.2, 0.2, 0.2, 1.0);
   Mat_Diffuse    : Array[0..3] Of TFloat32 = (0.2, 0.2, 0.2, 1.0);

   Light_Ambient  : Array[0..3] Of TFloat32 = (0.2, 0.2, 0.2, 1.0);
   Light_Diffuse  : Array[0..3] Of TFloat32 = (0.4, 0.4, 0.4, 1.0);

   Light_Position : Array[0..3] Of TFloat32 = (-1, 1, 0, 1.0);

VAR
   //OGLFont: TStdFont;
   OGLWindow, OGLWidth, OGLHeight: TuInt32;
   OGLZNear, OGLZFar: TInt32;
   OGLViewMatrix: TMatrix4D;

Procedure SetLight0; Forward;
//********************************************************************
//********************************************************************
Procedure SetGraphicWindow(AWindow: TuInt32);
Begin
   OGLWindow:= AWindow;
End;
//********************************************************************
Procedure ResizeGraphicWindow(AWidth,AHeight: TuInt32);
Var
   XMax,YMax: TFloat32;
Begin
   OGLWidth := AWidth;
   OGLHeight:= AHeight;
   glViewport(0,0,OGLWidth,OGLHeight);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity;

   YMax := Tan(45/2*PI/180)*OGLZNear;
   XMax := YMax*OGLWidth/OGLHeight;
   glFrustum(-XMax,+XMax,-YMax,+YMax,OGLZNear,OGLZFar);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;
End;
//********************************************************************
Procedure InitGraphicParams(AZNear,AZFar: TInt32);
Begin
   OGLZNear := AZNear;
   OGLZFar  := AZFar;

   glEnable(GL_CULL_FACE);
   glCullFace(GL_BACK);

   glEnable(GL_BLEND);
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

   glEnable(GL_DEPTH_TEST);
   glDepthFunc(GL_LEQUAL);

   glEnable(GL_Texture_2D); // Font!
   glEnable(GL_LIGHTING);
   SetLight0;

   glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);  // Filled
   //glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);  // WireFrame

   glShadeModel(GL_FLAT);
   //glShadeModel(GL_SMOOTH);

   glClearColor(0,0,0,1);
End;
//********************************************************************
Procedure CreateGraphicTools;
Begin
   //CharSet:= CreateCharSet(1);
   //OGLFont:= TStdFont.Create('OGLFont',18,SBLACK);
   //OGLFont.Load('FreeMono.ttf');
End;
//********************************************************************
Procedure FreeGraphicTools;
Begin
   //Finalize(CharSet);
   //OGLFont.Free;
End;
//********************************************************************
Procedure SetViewMatrix(AViewMatrix: TMatrix4D);
Begin
   OGLViewMatrix:= AViewMatrix;
End;
//********************************************************************
Procedure SetLight0;
Begin
   glMaterialfv(GL_FRONT, GL_SPECULAR,  @Mat_Specular[0]);
   glMaterialfv(GL_FRONT, GL_SHININESS, @Mat_Shininess[0]);
   glMaterialfv(GL_FRONT, GL_AMBIENT,   @Mat_Ambient[0]);
   glMaterialfv(GL_FRONT, GL_DIFFUSE,   @Mat_Diffuse[0]);

   glLightfv(GL_LIGHT0, GL_AMBIENT,  @Light_Ambient[0]);
   glLightfv(GL_LIGHT0, GL_DIFFUSE,  @Light_Diffuse[0]);
   glLightfv(GL_LIGHT0, GL_POSITION, @Light_Position[0]);

   glEnable(GL_COLOR_MATERIAL);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
End;
//********************************************************************
Procedure BeginRender;
Begin
   glClear(GL_COLOR_BUFFER_BIT Or GL_DEPTH_BUFFER_BIT);
End;
//********************************************************************
Procedure EndRender;
Begin
   {glMatrixMode(GL_PROJECTION);
   glLoadIdentity;
   glOrtho(0,AWidth,AHeight,0,+1,-1);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity;
   OGLFont.Print(5,5,100,30,'FPS: '+ IntToStr(FPS)); }

   osWindowSwapBuffers(OGLWindow);
End;
//********************************************************************
Procedure Render(AMatrix: TMatrix4D; AColor: T32bColor;
                 AVertexBuffer,ANormalBuffer: TVertexBuffer;
                 AIndexBuffer: TIndexBuffer);
Begin
   glLoadMatrixf(@OGLViewMatrix);
   glMultMatrixf(@AMatrix);

   glEnableClientState(GL_VERTEX_ARRAY);
   glEnableClientState(GL_NORMAL_ARRAY);

   glVertexPointer(3,GL_FLOAT,0,@AVertexBuffer[0]);
   glNormalPointer(GL_FLOAT,0,@ANormalBuffer[0]);

   With AColor Do glColor4ub(Red,Green,Blue,Alpha);

   glDrawElements(GL_TRIANGLES, Length(AIndexBuffer),
                        GL_UNSIGNED_SHORT, @AIndexBuffer[0]);

   glDisableClientState(GL_NORMAL_ARRAY);
   glDisableClientState(GL_VERTEX_ARRAY);
End;
//********************************************************************
//********************************************************************
End.
