//********************************************************************
PROGRAM KollisionTest;
//********************************************************************

USES
   SysUtils,
   Classes,
   dglOpenGL,
   StdBasicDefs,
   OSSupport,
   Math3D,
   GraphicSupport,
   Shape3D;

TYPE
   TEventHandler = Class(TObject)
      Done: TBool8;
      Procedure KeyDown(AWindow: TuInt32; AKey,PreviousKey: TInt32);
   End;

VAR
   MyEventHandler: TEventHandler;
   MainWindow, RenderContext: TuInt32;
   Left,Top,Width,Height: TInt32;
   StartTime,LocalTime,TimeCount,FrameCount,FPS: TInt64;

//********************************************************************
Procedure TEventHandler.KeyDown(AWindow: TuInt32;
                                AKey,PreviousKey: TInt32);
Begin
   If AKey = FK_ESCAPE
      Then Done:= TRUE;
End;
//********************************************************************
Procedure SetScene;
Var
   Triangle,Rectangle,Cube,Sphere: TShape3D;
   StartMatrix: TMatrix4D;
   Shift: TVector4D;
   Angle: TFloat32;
Begin
   With ShapeMan Do Begin
      Triangle:= CreateShape3D(st3Triangle,3,MBLUE);
      //Rectangle:= CreateShape3D(st3Rectangle,1,SYELLOW);
      //Cube:= CreateShape3D(st3CUBE,0.5,SORANGE);
      Sphere:= CreateShape3D(st3Sphere,1,SGREEN);
   End;

   // SPHERE --------------------------------------------------
   Angle:= 90;
   StartMatrix:= MatrixRotate(Angle,0,1,0);
   Shift:= VectorMultiply(StartMatrix[2],-5);
   StartMatrix[3]:= VectorAdd(StartMatrix[3],Shift);
   Sphere.Animate(StartMatrix,10000,0.1);

   // TRIANGLE ------------------------------------------------
   {Angle:= +30;
   StartMatrix:= MatrixRotate(Angle,0,1,0);
   Triangle.Matrix:= StartMatrix; }
End;
//********************************************************************
CONST
   DO_NOT_WAIT_FOR_MESSAGE = FALSE;
VAR
   MyRenderContext: TRenderContextInfo;
   ApplName: TString8;
Begin
   //*****************************************************************
   // INITIALIZE DISPLAY SYSTEM AND RENDERING CONTEXT
   With MyRenderContext Do Begin
      IsRGBA:= TRUE;
      IsDoubleBuffered:= TRUE;
      IsStereo:= FALSE;
      ColorBits  := 32;
      ZBits      := 24;
      StencilBits:= 8;
      AccumBits  := 0;
      AuxBuffers := 0;
      Level      := 0;
   End;
   ApplName:= 'SpeedTest';
   If osDisplayInitialize(ApplName,MyRenderContext) Then Begin
      Left  := 100;
      Top   := 100;
      Width := 500;
      Height:= 500;

      MainWindow:= osWindowCreate
         (wtStandard,ApplName,Left,Top,Width,Height);
      If MainWindow > 0 Then Begin

         If InitOpenGL Then Begin

            RenderContext:= osRenderContextCreate(MainWindow,0);
            osWindowDisplay(MainWindow,TRUE);
            osWindowMove(MainWindow,Left,Top);

            If RenderContext > 0 Then Begin

               //*****************************************************
               // INITIALIZATION
               ReadOpenGLCore;
               ReadImplementationProperties;
               SetGraphicWindow(MainWindow);

               ShapeMan:= TShapeMan.Create;
               ResizeGraphicWindow(Width,Height);
               SetScene;

               FPS:= 0;
               TimeCount:= 0;
               FrameCount:= 0;
               StartTime:= osClockTick;

               MyEventHandler:= TEventHandler.Create;
               With MyEventHandler Do Begin
                  Done:= FALSE;
                  EventPipeline.OnKeyDown:= KeyDown;
               End;
               //*****************************************************
               // MAINLOOP
               While Not MyEventHandler.Done Do Begin
                  osEventProcess(MainWindow,DO_NOT_WAIT_FOR_MESSAGE);

                  ShapeMan.Execute;

                  Sleep(60);
                  LocalTime:= osClockTick;
                  TimeCount:= LocalTime - StartTime;
                  Inc(FrameCount);

                  If TimeCount >= 1000 Then Begin
                     FPS:= FrameCount;
                     osWindowCaption(MainWindow,'FPS: '+ IntToStr(FPS));
                     StartTime:= osClockTick;
                     TimeCount:= 0;
                     FrameCount:= 0;
                  End;
               End;
               //*****************************************************
               // FINALIZATION
               MyEventHandler.Free;
               ShapeMan.Free;
               //*****************************************************
            End;
            osRenderContextDestroy(MainWindow,RenderContext);
         End;
         osWindowDestroy(MainWindow);
         osDisplayFinalize;
      End;
   End;
End.

