{ THIS FILE IS PART OF THE glPasGUI-PROJECT.

Copyright (c) 2009, Mag.Gertraude Trawnicek, Austria
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.
    * The name of the copyright holder must not be used to endorse
      or promote products derived from this software without specific
      prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND ANY
EXPRESS OR IMPLIED WARRANTIES,INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
}
//********************************************************************
Unit OSSupport;
// 1.Description: Wrapper for the operating system
// 2.Defined objects: None
// 3.Dependencies: the operating system API
// 4.Limitations: supported: 1.Win32, 2.Linux
// 5.Note: The following files are included this unit:
//         1. OSSupport.inc,
//         2. OSSupportWin32.inc,
//         3. OSSupportX11.inc
//********************************************************************
INTERFACE

USES
   SysUtils, Classes, dglOpenGL, StdBasicDefs;

CONST
{$IFDEF WIN32}
   StdDelim = '\';
   OSBarHeight = 30;
{$ELSE}
   StdDelim = '/';
   OSBarHeight = 0;
{$ENDIF}

   WINDOW_MIN_SIZE = 150;

//********************************************************************
// INPUT KEY CODES

   FK_LBUTTON = -1;         // VK_LBUTTON = 1;
   FK_RBUTTON = -2;         // VK_RBUTTON = 2;
   FK_CANCEL = -3;          // VK_CANCEL = 3;
   FK_MBUTTON = -4;         // VK_MBUTTON = 4;
   FK_BACK = -8;            // VK_BACK = 8;
   FK_TAB = -9;             // VK_TAB = 9;
   FK_CLEAR = -12;          // VK_CLEAR = 12;
   FK_RETURN = -13;         // VK_RETURN = 13;
   FK_SHIFT = -16;          // VK_SHIFT = 16;
   FK_CONTROL = -17;        // VK_CONTROL = 17;
   FK_ALT = -18;            // VK_MENU = 18;
   FK_PAUSE = -19;          // VK_PAUSE = 19;
   FK_CAPITAL = -20;        // VK_CAPITAL = 20;
   FK_CONVERT = -28;        // VK_CONVERT = 28;
   FK_NONCONVERT = -29;     // VK_NONCONVERT = 29;
   FK_ACCEPT = -30;         // VK_ACCEPT = 30;
   FK_MODECHANGE = -31;     // VK_MODECHANGE = 31;
   FK_ESCAPE = -27;         // VK_ESCAPE = 27;
   FK_SPACE = -$20;         // VK_SPACE = $20;
   FK_PRIOR = -33;          // VK_PRIOR = 33;
   FK_NEXT = -34;           // VK_NEXT = 34;
   FK_END = -35;            // VK_END = 35;
   FK_HOME = -36;           // VK_HOME = 36;
   FK_LEFT = -37;           // VK_LEFT = 37;
   FK_UP = -38;             // VK_UP = 38;
   FK_RIGHT = -39;          // VK_RIGHT = 39;
   FK_DOWN = -40;           // VK_DOWN = 40;
   FK_SELECT = -41;         // VK_SELECT = 41;
   FK_PRINT = -42;          // VK_PRINT = 42;
   FK_EXECUTE = -43;        // VK_EXECUTE = 43;
   FK_SNAPSHOT = -44;       // VK_SNAPSHOT = 44;
   FK_INSERT = -45;         // VK_INSERT = 45;
   FK_DELETE = -46;         // VK_DELETE = 46;
   FK_HELP = -47;           // VK_HELP = 47;
   FK_UNDO = -48;
      // VK_0 thru          // VK_9 are the same as ASCII '0' thru '9' ($30 - $39)
      // VK_A thru          // VK_Z are the same as ASCII 'A' thru 'Z' ($41 - $5A)
   FK_LWIN = -91;           // VK_LWIN = 91;
   FK_RWIN = -92;           // VK_RWIN = 92;
   FK_APPS = -93;           // VK_APPS = 93;
   FK_NUMPAD0 = -96;        // VK_NUMPAD0 = 96;
   FK_NUMPAD1 = -97;        // VK_NUMPAD1 = 97;
   FK_NUMPAD2 = -98;        // VK_NUMPAD2 = 98;
   FK_NUMPAD3 = -99;        // VK_NUMPAD3 = 99;
   FK_NUMPAD4 = -100;       // VK_NUMPAD4 = 100;
   FK_NUMPAD5 = -101;       // VK_NUMPAD5 = 101;
   FK_NUMPAD6 = -102;       // VK_NUMPAD6 = 102;
   FK_NUMPAD7 = -103;       // VK_NUMPAD7 = 103;
   FK_NUMPAD8 = -104;       // VK_NUMPAD8 = 104;
   FK_NUMPAD9 = -105;       // VK_NUMPAD9 = 105;
   FK_MULTIPLY = -106;      // VK_MULTIPLY = 106;
   FK_ADD = -107;           // VK_ADD = 107;
   FK_SEPARATOR = -108;     // VK_SEPARATOR = 108;
   FK_SUBTRACT = -109;      // VK_SUBTRACT = 109;
   FK_DECIMAL = -110;       // VK_DECIMAL = 110;
   FK_DIVIDE = -111;        // VK_DIVIDE = 111;
   FK_F1 = -112;            // VK_F1 = 112;
   FK_F2 = -113;            // VK_F2 = 113;
   FK_F3 = -114;            // VK_F3 = 114;
   FK_F4 = -115;            // VK_F4 = 115;
   FK_F5 = -116;            // VK_F5 = 116;
   FK_F6 = -117;            // VK_F6 = 117;
   FK_F7 = -118;            // VK_F7 = 118;
   FK_F8 = -119;            // VK_F8 = 119;
   FK_F9 = -120;            // VK_F9 = 120;
   FK_F10 = -121;           // VK_F10 = 121;
   FK_F11 = -122;           // VK_F11 = 122;
   FK_F12 = -123;           // VK_F12 = 123;
   FK_F13 = -124;           // VK_F13 = 124;
   FK_F14 = -125;           // VK_F14 = 125;
   FK_F15 = -126;           // VK_F15 = 126;
   FK_F16 = -127;           // VK_F16 = 127;
   FK_F17 = -128;           // VK_F17 = 128;
   FK_F18 = -129;           // VK_F18 = 129;
   FK_F19 = -130;           // VK_F19 = 130;
   FK_F20 = -131;           // VK_F20 = 131;
   FK_F21 = -132;           // VK_F21 = 132;
   FK_F22 = -133;           // VK_F22 = 133;
   FK_F23 = -134;           // VK_F23 = 134;
   FK_F24 = -135;           // VK_F24 = 135;
   FK_NUMLOCK = -144;       // VK_NUMLOCK = 144;
   FK_SCROLL = -145;        // VK_SCROLL = 145;
   // VK_L &      // VK_R - left and right Alt, Ctrl and Shift virtual keys.
   // Used only as parameters to GetAsyncKeyState() and GetKeyState().
   // No other API or message will distinguish left and right keys in this way.
   FK_LSHIFT = -160;        // VK_LSHIFT = 160;
   FK_RSHIFT = -161;        // VK_RSHIFT = 161;
   FK_LCONTROL = -162;      // VK_LCONTROL = 162;
   FK_RCONTROL = -163;      // VK_RCONTROL = 163;
   FK_LMENU = -164;         // VK_LMENU = 164;
   FK_RMENU = -165;         // VK_RMENU = 165;
   FK_PROCESSKEY = -229;    // VK_PROCESSKEY = 229;
   FK_ATTN = -246;          // VK_ATTN = 246;
   FK_CRSEL = -247;         // VK_CRSEL = 247;
   FK_EXSEL = -248;         // VK_EXSEL = 248;
   FK_EREOF = -249;         // VK_EREOF = 249;
   FK_PLAY = -250;          // VK_PLAY = 250;
   FK_ZOOM = -251;          // VK_ZOOM = 251;
   FK_NONAME = -252;        // VK_NONAME = 252;
   FK_PA1 = -253;           // VK_PA1 = 253;
   FK_OEM_CLEAR = -254;     // VK_OEM_CLEAR = 254;

   FK_BIGZ= -90;

//********************************************************************
TYPE
   OSException = Exception;

   TMouseImage = (miArrow, miHourGlass, miIBeam, miCross,
                  miMoving, miSizing);

   TWindowType = (wtPopup, wtStandard, {wtNative,} wtFullScreen);

   TWndChangeEvent = Procedure(AReceiver: TuInt32; X,Y: TInt32) Of Object;
   TWndNotifyEvent = Procedure(AReceiver: TuInt32) Of Object;
   TWndKeyEvent = Procedure(AReceiver: TuInt32; AKey,PreviousKey: TInt32) Of Object;
   TWndMouseBtnEvent = Procedure(AReceiver: TuInt32; X,Y,AddKeys: TInt32) Of Object;
   TWndMouseMoveEvent = Procedure(AReceiver: TuInt32; X,Y: TInt32) Of Object;
   TWndMouseWheelEvent = Procedure(AReceiver: TuInt32; X,Y,ADelta: TInt32) Of Object;

   TEventPipeline = Record
      OnClose         : TWndNotifyEvent;
      OnPaint         : TWndNotifyEvent;
      OnMove          : TWndChangeEvent;
      OnResize        : TWndChangeEvent;
      OnRestore       : TWndNotifyEvent;

      OnKeyDown       : TWndKeyEvent;
      OnKeyUp         : TWndKeyEvent;
      OnMouseBtnLDown : TWndMouseBtnEvent;
      OnMouseBtnLUp   : TWndMouseBtnEvent;
      OnMouseBtnMDown : TWndMouseBtnEvent;
      OnMouseBtnMUp   : TWndMouseBtnEvent;
      OnMouseBtnRDown : TWndMouseBtnEvent;
      OnMouseBtnRUp   : TWndMouseBtnEvent;
      OnMouseMove     : TWndMouseMoveEvent;
      OnMouseWheelDown: TWndMouseWheelEvent;
      OnMouseWheelUp  : TWndMouseWheelEvent;
   End;

   TVideoSettings = Record
      BitsPerPixel: TInt32;
      Width: TInt32;
      Height: TInt32;
      Frequency: TInt32;
   End;

   TRenderContextInfo = Record
      IsRGBA: TBool8;
      IsDoubleBuffered: TBool8;
      IsStereo: TBool8;

      ColorBits: TInt32;
      ZBits: TInt32;
      StencilBits: TInt32;
      AccumBits: TInt32;

      AuxBuffers: TInt32;

      Level: TInt32;
   End;

//********************************************************************
VAR
   EventPipeline: TEventPipeline;
//********************************************************************
// O P E R A T I N G   S Y S T E M   R O U T I N E S
//********************************************************************
// Initialize/Finalize                         // Unicode-Switch &&&&&
Function  osDisplayInitialize(IdentityString: TString8;
                  ARenderContextInfo: TRenderContextInfo): TBool8;
Procedure osDisplayFinalize;
Function  osDisplay: TPoint32; // This is a pointer to X11 XDisplay
                               // In MS-Windows the result is NIL
//--------------------------------------------------------------------
// Clock
Function osClockFrequency: TInt64;
Function osClockTick: TInt64;
//--------------------------------------------------------------------
// Copy & Paste                                // Unicode-Switch &&&&&
Procedure osCopyString(AWindow: TuInt32; AString: TString8);
Function  osPasteString(AWindow: TuInt32): TString8;
//--------------------------------------------------------------------
// Directories
Function  osRootDir: TStringList;
//--------------------------------------------------------------------
// Events
Function  osEventProcess(AWindow: TuInt32;
               WaitForMessage: TBool8): TBool8;
Procedure osEventTerminateApplication;
//--------------------------------------------------------------------
// Mouse   // necessary for MS-Windows only
Procedure osMouseCapture(AWindow: TuInt32);
Procedure osMouseCaptureRelease;
Procedure osMouseImageSet(AWindow: TuInt32; AMouseImage: TMouseImage);
Function  osMousePosition(AWindow: TuInt32): TPoint2D;
Procedure osMousePositionSet(AWindow: TuInt32; AX,AY: TInt32);
//--------------------------------------------------------------------
// Rendering Context
Function  osRenderContextCreate
               (AWindow,ASharedContext: TuInt32): TuInt32;
Function  osRenderContextDestroy
               (AWindow,ARenderContext: TuInt32): TBool8;
Function  osRenderContextActivate
               (AWindow,ARenderContext: TuInt32): TBool8;
Procedure osRenderContextDeactivate;
Function  osGetCurrentContext: TuInt32;
//--------------------------------------------------------------------
// Videomode
Function  osVideoSettings: TVideoSettings;
//--------------------------------------------------------------------
// Windows
Function  osWindowCreate(AWindowType: TWindowType;
               ACaption: TString8; // Unicode-Switch &&&&&
               ALeft,ATop,AWidth,AHeight: TInt32): TuInt32;
Procedure osWindowDestroy(AWindow: TuInt32);
Procedure osWindowCaption(AWindow: TuInt32; ACaption: TString8);
Procedure osWindowDisplay(AWindow: TuInt32; Visible: TBool8);
Procedure osWindowMaximize(AWindow: TuInt32);
Procedure osWindowMinimize(AWindow: TuInt32);
Procedure osWindowMove(AWindow: TuInt32; ALeft,ATop: TInt32);
Procedure osWindowResize(AWindow: TuInt32; AWidth,AHeight: TInt32);
Procedure osWindowRestore(AWindow: TuInt32;
               ALeft,ATop,AWidth,AHeight: TInt32);
Procedure osWindowSwapBuffers(AWindow: TuInt32);
//--------------------------------------------------------------------
// "Special" - Windows routines (for MS-Windows only)
Procedure osWindowSpecialMove(AWindow: TuInt32; MouseX,MouseY: TInt32);
Procedure osWindowSpecialResize(AWindow: TuInt32; MouseX,MouseY: TInt32);
//********************************************************************
IMPLEMENTATION
//********************************************************************
{$IFDEF WIN32}
   {$I OSSupportWin32.inc}
{$ELSE}
   {$I OSSupportX11.inc}
{$ENDIF}
//********************************************************************
End.

